package com.smith9.geiger.client

import com.smith9.geiger.client.beans.Duration

/**
 * Created: 7/06/11
 * @author david
 */
class BatchingGeigerClient implements BatchingDurationClient {
    DurationClient client
    List<Duration> durations = []
    Duration lastDurationCommitted
    int batchSize

    BatchingGeigerClient(int batchSize, DurationClient client) {
        this.client = client
        this.batchSize = batchSize
    }

    Duration getLastDurationCommitted() {
        return lastDurationCommitted
    }

    boolean batch(List<Duration> durationList) {
        durations.addAll(durationList)
        if (durations.size() >= batchSize) {
            send()
            return true
        }
        return false
    }

    boolean batchRemaining() {
        if (!durations.isEmpty()) {
            lastDurationCommitted = client.send(durations)
            durations.clear()
            return true
        }
        return false
    }

    private void send() {
        int durationCount = durations.size()
        int startIndex = 0
        int endIndex = 0
        while ((endIndex = startIndex + batchSize) <= durationCount) {
            List<Duration> batch = durations.subList(startIndex, endIndex)
            lastDurationCommitted = client.send(batch)
            startIndex = endIndex
        }
        //
        List<Duration> tail = new ArrayList(durations.subList(startIndex, durationCount))
        durations.clear()
        durations = tail
    }

    int size() {
        return durations.size()
    }
}
