package com.smith9.geiger.client

import com.smith9.geiger.client.beans.Duration
import org.apache.commons.httpclient.methods.PostMethod
import org.apache.commons.httpclient.HttpClient

import org.joda.time.DateTime

import groovy.xml.MarkupBuilder
import org.apache.commons.httpclient.HttpStatus
import org.slf4j.Logger
import org.slf4j.LoggerFactory

/**
 * Created: Mar 17, 2011
 * @author david
 */

class GeigerClient implements DurationClient {
    private static final Logger LOG = LoggerFactory.getLogger(GeigerClient.class)
    private URL url
    private long totalSent
    private long totalDuration
//    private long batchesSent
    private HttpClient client

    public static void main(String[] args) {
        List<Duration> durations = [new Duration(environment: "SOTL", product: "EasyDoc", subProduct: "PI", startTime: new DateTime(), duration: 12, measure: "descr", grouping: "1234567", reference: "123:413", failure: false, metadata: [start: "1", end: "5"]),
            new Duration(environment: "SOTL", product: "EasyDoc", subProduct: "PI", startTime: new DateTime(), duration: 9, measure: "descr2", grouping: "1234567", reference: "123:414", failure: false, metadata: [start: "1", end: "5", error: '$10&<$5'])]

        new GeigerClient(new URL("http://localhost:8080/geiger/duration")).addAll(durations)
    }

    public GeigerClient(URL url) {
        this.url = url
        client = new HttpClient()
//        batchesSent = 0
    }

    Duration send(List<Duration> durations) {
        long start = System.currentTimeMillis()
        String xml = createXml(durations)
        uploadDurations(xml)
        logTimeTaken(System.currentTimeMillis() - start, durations.size())
        return durations.last()
    }

    void logTimeTaken(Long duration, int count) {
        if(LOG.debugEnabled)
        {
            totalSent += count
            totalDuration += duration
            LOG.debug("Sent $count, 1 took ${duration/count/1000}s, $totalSent averaged ${totalDuration/totalSent/1000}s")
        }
    }

    private void uploadDurations(String xml) {
//        batchesSent++
//        cleanupClient()
        PostMethod post = new PostMethod(url.toString())
        try {
            post.addParameter("xml", xml)
            int status = client.executeMethod(post)
            if (status != org.apache.commons.httpclient.HttpStatus.SC_OK) {
                LOG.error("Error uploading durations to $url xml to post was: $xml")
                LOG.error("status code " + post.getStatusCode())
                LOG.error(post.getResponseBodyAsString())
            } else {
                // cleanup - see http://hc.apache.org/httpclient-3.x/tutorial.html
                cleanupResponseBody(post)
            }
        } finally {
            // cleanup - see http://hc.apache.org/httpclient-3.x/tutorial.html
            post.releaseConnection()
        }
    }

//    void cleanupClient() {
//        if(batchesSent % 100 == 0) {
//            LOG.debug("creating a new HttpClient to free memory/resources")
//            client = new HttpClient()
//        }
//    }

    private void cleanupResponseBody(PostMethod post) {
        InputStream response = post.getResponseBodyAsStream()
        BufferedReader bufferedResponse = new BufferedReader(new InputStreamReader(response));
        String line;
        while ((line = bufferedResponse.readLine()) != null) {
        }
        bufferedResponse.close();
    }

    private String createXml(List<Duration> durations) {
        StringWriter writer = new StringWriter()
        MarkupBuilder builder = new MarkupBuilder(writer)
        builder.durations() {
            durations.each { durationNode ->
                duration(toMap(durationNode)) {
                    metadata(durationNode.metadata)
                }
            }
        }
        return writer.toString()
    }

    static Map<String, String> toMap(Duration duration) {
        assert duration.startTime, "StartTime is mandatory"
        assert duration.measure, "measure is mandatory"
        Map<String, String> properties = [:]
        properties.put("environment", duration.environment)
        properties.put("product", duration.product)
        properties.put("subProduct", duration.subProduct)
        properties.put("startTime", duration.startTime.toString("yyyy-MM-dd HH:mm:ss.SSS"))
        properties.put("duration", duration.duration)
        properties.put("measure", duration.measure)
        properties.put("grouping", duration.grouping)
        properties.put("reference", duration.reference)
        properties.put("batch", duration.batch)
        properties.put("batchType", duration.batchType)
        properties.put("failure", duration.failure)
        return properties
    }
}